"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const graphql_1 = require("graphql");
const copy_to_clipboard_1 = __importDefault(require("copy-to-clipboard"));
const ExecuteButton_1 = require("./ExecuteButton");
const ImagePreview_1 = require("./ImagePreview");
const ToolbarButton_1 = require("./ToolbarButton");
const ToolbarGroup_1 = require("./ToolbarGroup");
const ToolbarMenu_1 = require("./ToolbarMenu");
const QueryEditor_1 = require("./QueryEditor");
const VariableEditor_1 = require("./VariableEditor");
const ResultViewer_1 = require("./ResultViewer");
const DocExplorer_1 = require("./DocExplorer");
const QueryHistory_1 = require("./QueryHistory");
const CodeMirrorSizer_1 = __importDefault(require("../utility/CodeMirrorSizer"));
const StorageAPI_1 = __importDefault(require("../utility/StorageAPI"));
const getQueryFacts_1 = __importDefault(require("../utility/getQueryFacts"));
const getSelectedOperationName_1 = __importDefault(require("../utility/getSelectedOperationName"));
const debounce_1 = __importDefault(require("../utility/debounce"));
const find_1 = __importDefault(require("../utility/find"));
const fillLeafs_1 = require("../utility/fillLeafs");
const elementPosition_1 = require("../utility/elementPosition");
const mergeAst_1 = __importDefault(require("../utility/mergeAst"));
const introspectionQueries_1 = require("../utility/introspectionQueries");
const DEFAULT_DOC_EXPLORER_WIDTH = 350;
const majorVersion = parseInt(react_1.default.version.slice(0, 2), 10);
if (majorVersion < 16) {
    throw Error([
        'GraphiQL 0.18.0 and after is not compatible with React 15 or below.',
        'If you are using a CDN source (jsdelivr, unpkg, etc), follow this example:',
        'https://github.com/graphql/graphiql/blob/master/examples/graphiql-cdn/index.html#L49',
    ].join('\n'));
}
class GraphiQL extends react_1.default.Component {
    constructor(props) {
        super(props);
        this._editorQueryID = 0;
        this.handleClickReference = (reference) => {
            this.setState({ docExplorerOpen: true }, () => {
                if (this.docExplorerComponent) {
                    this.docExplorerComponent.showDocForReference(reference);
                }
            });
        };
        this.handleRunQuery = (selectedOperationName) => {
            this._editorQueryID++;
            const queryID = this._editorQueryID;
            const editedQuery = this.autoCompleteLeafs() || this.state.query;
            const variables = this.state.variables;
            let operationName = this.state.operationName;
            if (selectedOperationName && selectedOperationName !== operationName) {
                operationName = selectedOperationName;
                this.handleEditOperationName(operationName);
            }
            try {
                this.setState({
                    isWaitingForResponse: true,
                    response: undefined,
                    operationName,
                });
                if (this._queryHistory) {
                    this._queryHistory.updateHistory(editedQuery, variables, operationName);
                }
                const subscription = this._fetchQuery(editedQuery, variables, operationName, (result) => {
                    if (queryID === this._editorQueryID) {
                        this.setState({
                            isWaitingForResponse: false,
                            response: GraphiQL.formatResult(result),
                        });
                    }
                });
                this.setState({ subscription });
            }
            catch (error) {
                this.setState({
                    isWaitingForResponse: false,
                    response: error.message,
                });
            }
        };
        this.handleStopQuery = () => {
            const subscription = this.state.subscription;
            this.setState({
                isWaitingForResponse: false,
                subscription: null,
            });
            if (subscription) {
                subscription.unsubscribe();
            }
        };
        this.handlePrettifyQuery = () => {
            const editor = this.getQueryEditor();
            const editorContent = editor?.getValue() ?? '';
            const prettifiedEditorContent = graphql_1.print(graphql_1.parse(editorContent));
            if (prettifiedEditorContent !== editorContent) {
                editor?.setValue(prettifiedEditorContent);
            }
            const variableEditor = this.getVariableEditor();
            const variableEditorContent = variableEditor?.getValue() ?? '';
            try {
                const prettifiedVariableEditorContent = JSON.stringify(JSON.parse(variableEditorContent), null, 2);
                if (prettifiedVariableEditorContent !== variableEditorContent) {
                    variableEditor?.setValue(prettifiedVariableEditorContent);
                }
            }
            catch {
            }
        };
        this.handleMergeQuery = () => {
            const editor = this.getQueryEditor();
            const query = editor.getValue();
            if (!query) {
                return;
            }
            const ast = graphql_1.parse(query);
            editor.setValue(graphql_1.print(mergeAst_1.default(ast)));
        };
        this.handleEditQuery = debounce_1.default(100, (value) => {
            const queryFacts = this._updateQueryFacts(value, this.state.operationName, this.state.operations, this.state.schema);
            this.setState({
                query: value,
                ...queryFacts,
            });
            if (this.props.onEditQuery) {
                return this.props.onEditQuery(value);
            }
        });
        this.handleCopyQuery = () => {
            const editor = this.getQueryEditor();
            const query = editor && editor.getValue();
            if (!query) {
                return;
            }
            copy_to_clipboard_1.default(query);
            if (this.props.onCopyQuery) {
                return this.props.onCopyQuery(query);
            }
        };
        this._updateQueryFacts = (query, operationName, prevOperations, schema) => {
            const queryFacts = getQueryFacts_1.default(schema, query);
            if (queryFacts) {
                const updatedOperationName = getSelectedOperationName_1.default(prevOperations, operationName, queryFacts.operations);
                const onEditOperationName = this.props.onEditOperationName;
                if (onEditOperationName &&
                    updatedOperationName &&
                    operationName !== updatedOperationName) {
                    onEditOperationName(updatedOperationName);
                }
                return {
                    operationName: updatedOperationName,
                    ...queryFacts,
                };
            }
        };
        this.handleEditVariables = (value) => {
            this.setState({ variables: value });
            if (this.props.onEditVariables) {
                this.props.onEditVariables(value);
            }
        };
        this.handleEditOperationName = (operationName) => {
            const onEditOperationName = this.props.onEditOperationName;
            if (onEditOperationName) {
                onEditOperationName(operationName);
            }
        };
        this.handleHintInformationRender = (elem) => {
            elem.addEventListener('click', this._onClickHintInformation);
            let onRemoveFn;
            elem.addEventListener('DOMNodeRemoved', (onRemoveFn = () => {
                elem.removeEventListener('DOMNodeRemoved', onRemoveFn);
                elem.removeEventListener('click', this._onClickHintInformation);
            }));
        };
        this.handleEditorRunQuery = () => {
            this._runQueryAtCursor();
        };
        this._onClickHintInformation = (event) => {
            if (event?.currentTarget &&
                'className' in event.currentTarget &&
                event.currentTarget.className === 'typeName') {
                const typeName = event.currentTarget.innerHTML;
                const schema = this.state.schema;
                if (schema) {
                    const type = schema.getType(typeName);
                    if (type) {
                        this.setState({ docExplorerOpen: true }, () => {
                            if (this.docExplorerComponent) {
                                this.docExplorerComponent.showDoc(type);
                            }
                        });
                    }
                }
            }
        };
        this.handleToggleDocs = () => {
            if (typeof this.props.onToggleDocs === 'function') {
                this.props.onToggleDocs(!this.state.docExplorerOpen);
            }
            this.setState({ docExplorerOpen: !this.state.docExplorerOpen });
        };
        this.handleToggleHistory = () => {
            if (typeof this.props.onToggleHistory === 'function') {
                this.props.onToggleHistory(!this.state.historyPaneOpen);
            }
            this.setState({ historyPaneOpen: !this.state.historyPaneOpen });
        };
        this.handleSelectHistoryQuery = (query, variables, operationName) => {
            if (query) {
                this.handleEditQuery(query);
            }
            if (variables) {
                this.handleEditVariables(variables);
            }
            if (operationName) {
                this.handleEditOperationName(operationName);
            }
        };
        this.handleResizeStart = (downEvent) => {
            if (!this._didClickDragBar(downEvent)) {
                return;
            }
            downEvent.preventDefault();
            const offset = downEvent.clientX - elementPosition_1.getLeft(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                const editorBar = this.editorBarComponent;
                const leftSize = moveEvent.clientX - elementPosition_1.getLeft(editorBar) - offset;
                const rightSize = editorBar.clientWidth - leftSize;
                this.setState({ editorFlex: leftSize / rightSize });
            };
            let onMouseUp = () => {
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        this.handleResetResize = () => {
            this.setState({ editorFlex: 1 });
        };
        this.handleDocsResizeStart = downEvent => {
            downEvent.preventDefault();
            const hadWidth = this.state.docExplorerWidth;
            const offset = downEvent.clientX - elementPosition_1.getLeft(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                const app = this.graphiqlContainer;
                const cursorPos = moveEvent.clientX - elementPosition_1.getLeft(app) - offset;
                const docsSize = app.clientWidth - cursorPos;
                if (docsSize < 100) {
                    this.setState({ docExplorerOpen: false });
                }
                else {
                    this.setState({
                        docExplorerOpen: true,
                        docExplorerWidth: Math.min(docsSize, 650),
                    });
                }
            };
            let onMouseUp = () => {
                if (!this.state.docExplorerOpen) {
                    this.setState({ docExplorerWidth: hadWidth });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        this.handleDocsResetResize = () => {
            this.setState({
                docExplorerWidth: DEFAULT_DOC_EXPLORER_WIDTH,
            });
        };
        this.handleVariableResizeStart = downEvent => {
            downEvent.preventDefault();
            let didMove = false;
            const wasOpen = this.state.variableEditorOpen;
            const hadHeight = this.state.variableEditorHeight;
            const offset = downEvent.clientY - elementPosition_1.getTop(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                didMove = true;
                const editorBar = this.editorBarComponent;
                const topSize = moveEvent.clientY - elementPosition_1.getTop(editorBar) - offset;
                const bottomSize = editorBar.clientHeight - topSize;
                if (bottomSize < 60) {
                    this.setState({
                        variableEditorOpen: false,
                        variableEditorHeight: hadHeight,
                    });
                }
                else {
                    this.setState({
                        variableEditorOpen: true,
                        variableEditorHeight: bottomSize,
                    });
                }
            };
            let onMouseUp = () => {
                if (!didMove) {
                    this.setState({ variableEditorOpen: !wasOpen });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        if (typeof props.fetcher !== 'function') {
            throw new TypeError('GraphiQL requires a fetcher function.');
        }
        this._storage = new StorageAPI_1.default(props.storage);
        const query = props.query !== undefined
            ? props.query
            : this._storage.get('query')
                ? this._storage.get('query')
                : props.defaultQuery !== undefined
                    ? props.defaultQuery
                    : defaultQuery;
        const queryFacts = getQueryFacts_1.default(props.schema, query);
        const variables = props.variables !== undefined
            ? props.variables
            : this._storage.get('variables');
        const operationName = props.operationName !== undefined
            ? props.operationName
            : getSelectedOperationName_1.default(undefined, this._storage.get('operationName'), queryFacts && queryFacts.operations);
        let docExplorerOpen = props.docExplorerOpen || false;
        if (this._storage.get('docExplorerOpen')) {
            docExplorerOpen = this._storage.get('docExplorerOpen') === 'true';
        }
        const variableEditorOpen = props.defaultVariableEditorOpen !== undefined
            ? props.defaultVariableEditorOpen
            : Boolean(variables);
        this.state = {
            schema: props.schema,
            query,
            variables: variables,
            operationName,
            docExplorerOpen,
            response: props.response,
            editorFlex: Number(this._storage.get('editorFlex')) || 1,
            variableEditorOpen,
            variableEditorHeight: Number(this._storage.get('variableEditorHeight')) || 200,
            historyPaneOpen: this._storage.get('historyPaneOpen') === 'true' || false,
            docExplorerWidth: Number(this._storage.get('docExplorerWidth')) ||
                DEFAULT_DOC_EXPLORER_WIDTH,
            isWaitingForResponse: false,
            subscription: null,
            ...queryFacts,
        };
        if (typeof window === 'object') {
            window.addEventListener('beforeunload', () => this.componentWillUnmount());
        }
    }
    static formatResult(result) {
        return JSON.stringify(result, null, 2);
    }
    static formatError(rawError) {
        const result = Array.isArray(rawError)
            ? rawError.map(formatSingleError)
            : formatSingleError(rawError);
        return JSON.stringify(result, null, 2);
    }
    componentDidMount() {
        if (this.state.schema === undefined) {
            this.fetchSchema();
        }
        this.codeMirrorSizer = new CodeMirrorSizer_1.default();
        global.g = this;
    }
    UNSAFE_componentWillReceiveProps(nextProps) {
        let nextSchema = this.state.schema;
        let nextQuery = this.state.query;
        let nextVariables = this.state.variables;
        let nextOperationName = this.state.operationName;
        let nextResponse = this.state.response;
        if (nextProps.schema !== undefined) {
            nextSchema = nextProps.schema;
        }
        if (nextProps.query !== undefined) {
            nextQuery = nextProps.query;
        }
        if (nextProps.variables !== undefined) {
            nextVariables = nextProps.variables;
        }
        if (nextProps.operationName !== undefined) {
            nextOperationName = nextProps.operationName;
        }
        if (nextProps.response !== undefined) {
            nextResponse = nextProps.response;
        }
        if (nextQuery &&
            nextSchema &&
            (nextSchema !== this.state.schema ||
                nextQuery !== this.state.query ||
                nextOperationName !== this.state.operationName)) {
            const updatedQueryAttributes = this._updateQueryFacts(nextQuery, nextOperationName, this.state.operations, nextSchema);
            if (updatedQueryAttributes !== undefined) {
                nextOperationName = updatedQueryAttributes.operationName;
                this.setState(updatedQueryAttributes);
            }
        }
        if (nextProps.schema === undefined &&
            nextProps.fetcher !== this.props.fetcher) {
            nextSchema = undefined;
        }
        this.setState({
            schema: nextSchema,
            query: nextQuery,
            variables: nextVariables,
            operationName: nextOperationName,
            response: nextResponse,
        }, () => {
            if (this.state.schema === undefined) {
                if (this.docExplorerComponent) {
                    this.docExplorerComponent.reset();
                }
                this.fetchSchema();
            }
        });
    }
    componentDidUpdate() {
        this.codeMirrorSizer.updateSizes([
            this.queryEditorComponent,
            this.variableEditorComponent,
            this.resultComponent,
        ]);
    }
    componentWillUnmount() {
        if (this.state.query) {
            this._storage.set('query', this.state.query);
        }
        if (this.state.variables) {
            this._storage.set('variables', this.state.variables);
        }
        if (this.state.operationName) {
            this._storage.set('operationName', this.state.operationName);
        }
        this._storage.set('editorFlex', JSON.stringify(this.state.editorFlex));
        this._storage.set('variableEditorHeight', JSON.stringify(this.state.variableEditorHeight));
        this._storage.set('docExplorerWidth', JSON.stringify(this.state.docExplorerWidth));
        this._storage.set('docExplorerOpen', JSON.stringify(this.state.docExplorerOpen));
        this._storage.set('historyPaneOpen', JSON.stringify(this.state.historyPaneOpen));
    }
    render() {
        const children = react_1.default.Children.toArray(this.props.children);
        const logo = find_1.default(children, child => isChildComponentType(child, GraphiQL.Logo)) || <GraphiQL.Logo />;
        const toolbar = find_1.default(children, child => isChildComponentType(child, GraphiQL.Toolbar)) || (<GraphiQL.Toolbar>
        <ToolbarButton_1.ToolbarButton onClick={this.handlePrettifyQuery} title="Prettify Query (Shift-Ctrl-P)" label="Prettify"/>
        <ToolbarButton_1.ToolbarButton onClick={this.handleMergeQuery} title="Merge Query (Shift-Ctrl-M)" label="Merge"/>
        <ToolbarButton_1.ToolbarButton onClick={this.handleCopyQuery} title="Copy Query (Shift-Ctrl-C)" label="Copy"/>
        <ToolbarButton_1.ToolbarButton onClick={this.handleToggleHistory} title="Show History" label="History"/>
      </GraphiQL.Toolbar>);
        const footer = find_1.default(children, child => isChildComponentType(child, GraphiQL.Footer));
        const queryWrapStyle = {
            WebkitFlex: this.state.editorFlex,
            flex: this.state.editorFlex,
        };
        const docWrapStyle = {
            display: 'block',
            width: this.state.docExplorerWidth,
        };
        const docExplorerWrapClasses = 'docExplorerWrap' +
            (this.state.docExplorerWidth < 200 ? ' doc-explorer-narrow' : '');
        const historyPaneStyle = {
            display: this.state.historyPaneOpen ? 'block' : 'none',
            width: '230px',
            zIndex: 7,
        };
        const variableOpen = this.state.variableEditorOpen;
        const variableStyle = {
            height: variableOpen ? this.state.variableEditorHeight : undefined,
        };
        return (<div ref={n => {
            this.graphiqlContainer = n;
        }} className="graphiql-container">
        <div className="historyPaneWrap" style={historyPaneStyle}>
          <QueryHistory_1.QueryHistory ref={node => {
            this._queryHistory = node;
        }} operationName={this.state.operationName} query={this.state.query} variables={this.state.variables} onSelectQuery={this.handleSelectHistoryQuery} storage={this._storage} queryID={this._editorQueryID}>
            <button className="docExplorerHide" onClick={this.handleToggleHistory} aria-label="Close History">
              {'\u2715'}
            </button>
          </QueryHistory_1.QueryHistory>
        </div>
        <div className="editorWrap">
          <div className="topBarWrap">
            <div className="topBar">
              {logo}
              <ExecuteButton_1.ExecuteButton isRunning={Boolean(this.state.subscription)} onRun={this.handleRunQuery} onStop={this.handleStopQuery} operations={this.state.operations}/>
              {toolbar}
            </div>
            {!this.state.docExplorerOpen && (<button className="docExplorerShow" onClick={this.handleToggleDocs} aria-label="Open Documentation Explorer">
                {'Docs'}
              </button>)}
          </div>
          <div ref={n => {
            this.editorBarComponent = n;
        }} className="editorBar" onDoubleClick={this.handleResetResize} onMouseDown={this.handleResizeStart}>
            <div className="queryWrap" style={queryWrapStyle}>
              <QueryEditor_1.QueryEditor ref={n => {
            this.queryEditorComponent = n;
        }} schema={this.state.schema} value={this.state.query} onEdit={this.handleEditQuery} onHintInformationRender={this.handleHintInformationRender} onClickReference={this.handleClickReference} onCopyQuery={this.handleCopyQuery} onPrettifyQuery={this.handlePrettifyQuery} onMergeQuery={this.handleMergeQuery} onRunQuery={this.handleEditorRunQuery} editorTheme={this.props.editorTheme} readOnly={this.props.readOnly}/>
              <section className="variable-editor" style={variableStyle} aria-label="Query Variables">
                <div className="variable-editor-title" id="variable-editor-title" style={{
            cursor: variableOpen ? 'row-resize' : 'n-resize',
        }} onMouseDown={this.handleVariableResizeStart}>
                  {'Query Variables'}
                </div>
                <VariableEditor_1.VariableEditor ref={n => {
            this.variableEditorComponent = n;
        }} value={this.state.variables} variableToType={this.state.variableToType} onEdit={this.handleEditVariables} onHintInformationRender={this.handleHintInformationRender} onPrettifyQuery={this.handlePrettifyQuery} onMergeQuery={this.handleMergeQuery} onRunQuery={this.handleEditorRunQuery} editorTheme={this.props.editorTheme} readOnly={this.props.readOnly}/>
              </section>
            </div>
            <div className="resultWrap">
              {this.state.isWaitingForResponse && (<div className="spinner-container">
                  <div className="spinner"/>
                </div>)}
              <ResultViewer_1.ResultViewer registerRef={n => {
            this.resultViewerElement = n;
        }} ref={c => {
            this.resultComponent = c;
        }} value={this.state.response} editorTheme={this.props.editorTheme} ResultsTooltip={this.props.ResultsTooltip} ImagePreview={ImagePreview_1.ImagePreview}/>
              {footer}
            </div>
          </div>
        </div>
        {this.state.docExplorerOpen && (<div className={docExplorerWrapClasses} style={docWrapStyle}>
            <div className="docExplorerResizer" onDoubleClick={this.handleDocsResetResize} onMouseDown={this.handleDocsResizeStart}/>
            <DocExplorer_1.DocExplorer ref={c => {
            this.docExplorerComponent = c;
        }} schema={this.state.schema}>
              <button className="docExplorerHide" onClick={this.handleToggleDocs} aria-label="Close Documentation Explorer">
                {'\u2715'}
              </button>
            </DocExplorer_1.DocExplorer>
          </div>)}
      </div>);
    }
    getQueryEditor() {
        if (this.queryEditorComponent) {
            return this.queryEditorComponent.getCodeMirror();
        }
    }
    getVariableEditor() {
        if (this.variableEditorComponent) {
            return this.variableEditorComponent.getCodeMirror();
        }
        return null;
    }
    refresh() {
        if (this.queryEditorComponent) {
            this.queryEditorComponent.getCodeMirror().refresh();
        }
        if (this.variableEditorComponent) {
            this.variableEditorComponent.getCodeMirror().refresh();
        }
        if (this.resultComponent) {
            this.resultComponent.getCodeMirror().refresh();
        }
    }
    autoCompleteLeafs() {
        const { insertions, result } = fillLeafs_1.fillLeafs(this.state.schema, this.state.query, this.props.getDefaultFieldNames);
        if (insertions && insertions.length > 0) {
            const editor = this.getQueryEditor();
            if (editor) {
                editor.operation(() => {
                    const cursor = editor.getCursor();
                    const cursorIndex = editor.indexFromPos(cursor);
                    editor.setValue(result || '');
                    let added = 0;
                    const markers = insertions.map(({ index, string }) => editor.markText(editor.posFromIndex(index + added), editor.posFromIndex(index + (added += string.length)), {
                        className: 'autoInsertedLeaf',
                        clearOnEnter: true,
                        title: 'Automatically added leaf fields',
                    }));
                    setTimeout(() => markers.forEach(marker => marker.clear()), 7000);
                    let newCursorIndex = cursorIndex;
                    insertions.forEach(({ index, string }) => {
                        if (index < cursorIndex) {
                            newCursorIndex += string.length;
                        }
                    });
                    editor.setCursor(editor.posFromIndex(newCursorIndex));
                });
            }
        }
        return result;
    }
    fetchSchema() {
        const fetcher = this.props.fetcher;
        const fetch = observableToPromise(fetcher({
            query: introspectionQueries_1.introspectionQuery,
            operationName: introspectionQueries_1.introspectionQueryName,
        }));
        if (!isPromise(fetch)) {
            this.setState({
                response: 'Fetcher did not return a Promise for introspection.',
            });
            return;
        }
        fetch
            .then(result => {
            if (typeof result !== 'string' && result.data) {
                return result;
            }
            const fetch2 = observableToPromise(fetcher({
                query: introspectionQueries_1.introspectionQuerySansSubscriptions,
                operationName: introspectionQueries_1.introspectionQueryName,
            }));
            if (!isPromise(fetch)) {
                throw new Error('Fetcher did not return a Promise for introspection.');
            }
            return fetch2;
        })
            .then(result => {
            if (this.state.schema !== undefined) {
                return;
            }
            if (typeof result !== 'string' && result && result.data) {
                const schema = graphql_1.buildClientSchema(result.data);
                const queryFacts = getQueryFacts_1.default(schema, this.state.query);
                this.setState({ schema, ...queryFacts });
            }
            else {
                const responseString = typeof result === 'string' ? result : GraphiQL.formatResult(result);
                this.setState({
                    schema: undefined,
                    response: responseString,
                });
            }
        })
            .catch(error => {
            this.setState({
                schema: undefined,
                response: error ? GraphiQL.formatError(error) : undefined,
            });
        });
    }
    _fetchQuery(query, variables, operationName, cb) {
        const fetcher = this.props.fetcher;
        let jsonVariables = null;
        try {
            jsonVariables =
                variables && variables.trim() !== '' ? JSON.parse(variables) : null;
        }
        catch (error) {
            throw new Error(`Variables are invalid JSON: ${error.message}.`);
        }
        if (typeof jsonVariables !== 'object') {
            throw new Error('Variables are not a JSON object.');
        }
        const fetch = fetcher({
            query,
            variables: jsonVariables,
            operationName,
        });
        if (isPromise(fetch)) {
            fetch.then(cb).catch(error => {
                this.setState({
                    isWaitingForResponse: false,
                    response: error ? GraphiQL.formatError(error) : undefined,
                });
            });
        }
        else if (isObservable(fetch)) {
            const subscription = fetch.subscribe({
                next: cb,
                error: (error) => {
                    this.setState({
                        isWaitingForResponse: false,
                        response: error ? GraphiQL.formatError(error) : undefined,
                        subscription: null,
                    });
                },
                complete: () => {
                    this.setState({
                        isWaitingForResponse: false,
                        subscription: null,
                    });
                },
            });
            return subscription;
        }
        else {
            throw new Error('Fetcher did not return Promise or Observable.');
        }
    }
    _runQueryAtCursor() {
        if (this.state.subscription) {
            this.handleStopQuery();
            return;
        }
        let operationName;
        const operations = this.state.operations;
        if (operations) {
            const editor = this.getQueryEditor();
            if (editor && editor.hasFocus()) {
                const cursor = editor.getCursor();
                const cursorIndex = editor.indexFromPos(cursor);
                for (let i = 0; i < operations.length; i++) {
                    const operation = operations[i];
                    if (operation.loc &&
                        operation.loc.start <= cursorIndex &&
                        operation.loc.end >= cursorIndex) {
                        operationName = operation.name && operation.name.value;
                        break;
                    }
                }
            }
        }
        this.handleRunQuery(operationName);
    }
    _didClickDragBar(event) {
        if (event.button !== 0 || event.ctrlKey) {
            return false;
        }
        let target = event.target;
        if (target.className.indexOf('CodeMirror-gutter') !== 0) {
            return false;
        }
        const resultWindow = this.resultViewerElement;
        while (target) {
            if (target === resultWindow) {
                return true;
            }
            target = target.parentNode;
        }
        return false;
    }
}
exports.GraphiQL = GraphiQL;
GraphiQL.Logo = GraphiQLLogo;
GraphiQL.Toolbar = GraphiQLToolbar;
GraphiQL.Footer = GraphiQLFooter;
GraphiQL.QueryEditor = QueryEditor_1.QueryEditor;
GraphiQL.VariableEditor = VariableEditor_1.VariableEditor;
GraphiQL.ResultViewer = ResultViewer_1.ResultViewer;
GraphiQL.Button = ToolbarButton_1.ToolbarButton;
GraphiQL.ToolbarButton = ToolbarButton_1.ToolbarButton;
GraphiQL.Group = ToolbarGroup_1.ToolbarGroup;
GraphiQL.Menu = ToolbarMenu_1.ToolbarMenu;
GraphiQL.MenuItem = ToolbarMenu_1.ToolbarMenuItem;
function GraphiQLLogo(props) {
    return (<div className="title">
      {props.children || (<span>
          {'Graph'}
          <em>{'i'}</em>
          {'QL'}
        </span>)}
    </div>);
}
GraphiQLLogo.displayName = 'GraphiQLLogo';
function GraphiQLToolbar(props) {
    return (<div className="toolbar" role="toolbar" aria-label="Editor Commands">
      {props.children}
    </div>);
}
GraphiQLToolbar.displayName = 'GraphiQLToolbar';
function GraphiQLFooter(props) {
    return <div className="footer">{props.children}</div>;
}
GraphiQLFooter.displayName = 'GraphiQLFooter';
const formatSingleError = (error) => ({
    ...error,
    message: error.message,
    stack: error.stack,
});
const defaultQuery = `# Welcome to GraphiQL
#
# GraphiQL is an in-browser tool for writing, validating, and
# testing GraphQL queries.
#
# Type queries into this side of the screen, and you will see intelligent
# typeaheads aware of the current GraphQL type schema and live syntax and
# validation errors highlighted within the text.
#
# GraphQL queries typically start with a "{" character. Lines that start
# with a # are ignored.
#
# An example GraphQL query might look like:
#
#     {
#       field(arg: "value") {
#         subField
#       }
#     }
#
# Keyboard shortcuts:
#
#  Prettify Query:  Shift-Ctrl-P (or press the prettify button above)
#
#     Merge Query:  Shift-Ctrl-M (or press the merge button above)
#
#       Run Query:  Ctrl-Enter (or press the play button above)
#
#   Auto Complete:  Ctrl-Space (or just start typing)
#

`;
function isPromise(value) {
    return typeof value === 'object' && typeof value.then === 'function';
}
function observableToPromise(observable) {
    if (!isObservable(observable)) {
        return observable;
    }
    return new Promise((resolve, reject) => {
        const subscription = observable.subscribe(v => {
            resolve(v);
            subscription.unsubscribe();
        }, reject, () => {
            reject(new Error('no value resolved'));
        });
    });
}
function isObservable(value) {
    return (typeof value === 'object' &&
        'subscribe' in value &&
        typeof value.subscribe === 'function');
}
function isChildComponentType(child, component) {
    if (child?.type?.displayName &&
        child.type.displayName === component.displayName) {
        return true;
    }
    return child.type === component;
}
//# sourceMappingURL=GraphiQL.jsx.map