"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_1 = __importDefault(require("react"));
const markdown_it_1 = __importDefault(require("markdown-it"));
const normalizeWhitespace_1 = require("../utility/normalizeWhitespace");
const onHasCompletion_1 = __importDefault(require("../utility/onHasCompletion"));
const commonKeys_1 = __importDefault(require("../utility/commonKeys"));
const md = new markdown_it_1.default();
const AUTO_COMPLETE_AFTER_KEY = /^[a-zA-Z0-9_@(]$/;
class QueryEditor extends react_1.default.Component {
    constructor(props) {
        super(props);
        this.editor = null;
        this.ignoreChangeEvent = false;
        this._node = null;
        this._onKeyUp = (_cm, event) => {
            if (AUTO_COMPLETE_AFTER_KEY.test(event.key) && this.editor) {
                this.editor.execCommand('autocomplete');
            }
        };
        this._onEdit = () => {
            if (!this.ignoreChangeEvent && this.editor) {
                this.cachedValue = this.editor.getValue();
                if (this.props.onEdit) {
                    this.props.onEdit(this.cachedValue);
                }
            }
        };
        this._onHasCompletion = (cm, data) => {
            onHasCompletion_1.default(cm, data, this.props.onHintInformationRender);
        };
        this.cachedValue = props.value || '';
    }
    componentDidMount() {
        const CodeMirror = require('codemirror');
        require('codemirror/addon/hint/show-hint');
        require('codemirror/addon/comment/comment');
        require('codemirror/addon/edit/matchbrackets');
        require('codemirror/addon/edit/closebrackets');
        require('codemirror/addon/fold/foldgutter');
        require('codemirror/addon/fold/brace-fold');
        require('codemirror/addon/search/search');
        require('codemirror/addon/search/searchcursor');
        require('codemirror/addon/search/jump-to-line');
        require('codemirror/addon/dialog/dialog');
        require('codemirror/addon/lint/lint');
        require('codemirror/keymap/sublime');
        require('codemirror-graphql/hint');
        require('codemirror-graphql/lint');
        require('codemirror-graphql/info');
        require('codemirror-graphql/jump');
        require('codemirror-graphql/mode');
        const editor = (this.editor = CodeMirror(this._node, {
            value: this.props.value || '',
            lineNumbers: true,
            tabSize: 2,
            mode: 'graphql',
            theme: this.props.editorTheme || 'graphiql',
            keyMap: 'sublime',
            autoCloseBrackets: true,
            matchBrackets: true,
            showCursorWhenSelecting: true,
            readOnly: this.props.readOnly ? 'nocursor' : false,
            foldGutter: {
                minFoldSize: 4,
            },
            lint: {
                schema: this.props.schema,
            },
            hintOptions: {
                schema: this.props.schema,
                closeOnUnfocus: false,
                completeSingle: false,
                container: this._node,
            },
            info: {
                schema: this.props.schema,
                renderDescription: (text) => md.render(text),
                onClick: (reference) => this.props.onClickReference && this.props.onClickReference(reference),
            },
            jump: {
                schema: this.props.schema,
                onClick: (reference) => this.props.onClickReference && this.props.onClickReference(reference),
            },
            gutters: ['CodeMirror-linenumbers', 'CodeMirror-foldgutter'],
            extraKeys: {
                'Cmd-Space': () => editor.showHint({ completeSingle: true, container: this._node }),
                'Ctrl-Space': () => editor.showHint({ completeSingle: true, container: this._node }),
                'Alt-Space': () => editor.showHint({ completeSingle: true, container: this._node }),
                'Shift-Space': () => editor.showHint({ completeSingle: true, container: this._node }),
                'Shift-Alt-Space': () => editor.showHint({ completeSingle: true, container: this._node }),
                'Cmd-Enter': () => {
                    if (this.props.onRunQuery) {
                        this.props.onRunQuery();
                    }
                },
                'Ctrl-Enter': () => {
                    if (this.props.onRunQuery) {
                        this.props.onRunQuery();
                    }
                },
                'Shift-Ctrl-C': () => {
                    if (this.props.onCopyQuery) {
                        this.props.onCopyQuery();
                    }
                },
                'Shift-Ctrl-P': () => {
                    if (this.props.onPrettifyQuery) {
                        this.props.onPrettifyQuery();
                    }
                },
                'Shift-Ctrl-F': () => {
                    if (this.props.onPrettifyQuery) {
                        this.props.onPrettifyQuery();
                    }
                },
                'Shift-Ctrl-M': () => {
                    if (this.props.onMergeQuery) {
                        this.props.onMergeQuery();
                    }
                },
                ...commonKeys_1.default,
                'Cmd-S': () => {
                    if (this.props.onRunQuery) {
                    }
                },
                'Ctrl-S': () => {
                    if (this.props.onRunQuery) {
                    }
                },
            },
        }));
        if (editor) {
            editor.on('change', this._onEdit);
            editor.on('keyup', this._onKeyUp);
            editor.on('hasCompletion', this._onHasCompletion);
            editor.on('beforeChange', this._onBeforeChange);
        }
    }
    componentDidUpdate(prevProps) {
        const CodeMirror = require('codemirror');
        this.ignoreChangeEvent = true;
        if (this.props.schema !== prevProps.schema && this.editor) {
            this.editor.options.lint.schema = this.props.schema;
            this.editor.options.hintOptions.schema = this.props.schema;
            this.editor.options.info.schema = this.props.schema;
            this.editor.options.jump.schema = this.props.schema;
            CodeMirror.signal(this.editor, 'change', this.editor);
        }
        if (this.props.value !== prevProps.value &&
            this.props.value !== this.cachedValue &&
            this.editor) {
            this.cachedValue = this.props.value;
            this.editor.setValue(this.props.value);
        }
        this.ignoreChangeEvent = false;
    }
    componentWillUnmount() {
        if (this.editor) {
            this.editor.off('change', this._onEdit);
            this.editor.off('keyup', this._onKeyUp);
            this.editor.off('hasCompletion', this._onHasCompletion);
            this.editor = null;
        }
    }
    render() {
        return (<section className="query-editor" aria-label="Query Editor" ref={node => {
            this._node = node;
        }}/>);
    }
    getCodeMirror() {
        return this.editor;
    }
    getClientHeight() {
        return this._node && this._node.clientHeight;
    }
    _onBeforeChange(_instance, change) {
        if (change.origin === 'paste') {
            const text = change.text.map(normalizeWhitespace_1.normalizeWhitespace);
            change.update(change.from, change.to, text);
        }
    }
}
exports.QueryEditor = QueryEditor;
//# sourceMappingURL=QueryEditor.jsx.map