import React from 'react';
import { buildClientSchema, parse, print, } from 'graphql';
import copyToClipboard from 'copy-to-clipboard';
import { ExecuteButton } from './ExecuteButton';
import { ImagePreview } from './ImagePreview';
import { ToolbarButton } from './ToolbarButton';
import { ToolbarGroup } from './ToolbarGroup';
import { ToolbarMenu, ToolbarMenuItem } from './ToolbarMenu';
import { QueryEditor } from './QueryEditor';
import { VariableEditor } from './VariableEditor';
import { ResultViewer } from './ResultViewer';
import { DocExplorer } from './DocExplorer';
import { QueryHistory } from './QueryHistory';
import CodeMirrorSizer from '../utility/CodeMirrorSizer';
import StorageAPI from '../utility/StorageAPI';
import getQueryFacts from '../utility/getQueryFacts';
import getSelectedOperationName from '../utility/getSelectedOperationName';
import debounce from '../utility/debounce';
import find from '../utility/find';
import { fillLeafs } from '../utility/fillLeafs';
import { getLeft, getTop } from '../utility/elementPosition';
import mergeAST from '../utility/mergeAst';
import { introspectionQuery, introspectionQueryName, introspectionQuerySansSubscriptions, } from '../utility/introspectionQueries';
const DEFAULT_DOC_EXPLORER_WIDTH = 350;
const majorVersion = parseInt(React.version.slice(0, 2), 10);
if (majorVersion < 16) {
    throw Error([
        'GraphiQL 0.18.0 and after is not compatible with React 15 or below.',
        'If you are using a CDN source (jsdelivr, unpkg, etc), follow this example:',
        'https://github.com/graphql/graphiql/blob/master/examples/graphiql-cdn/index.html#L49',
    ].join('\n'));
}
export class GraphiQL extends React.Component {
    constructor(props) {
        super(props);
        this._editorQueryID = 0;
        this.handleClickReference = (reference) => {
            this.setState({ docExplorerOpen: true }, () => {
                if (this.docExplorerComponent) {
                    this.docExplorerComponent.showDocForReference(reference);
                }
            });
        };
        this.handleRunQuery = (selectedOperationName) => {
            this._editorQueryID++;
            const queryID = this._editorQueryID;
            const editedQuery = this.autoCompleteLeafs() || this.state.query;
            const variables = this.state.variables;
            let operationName = this.state.operationName;
            if (selectedOperationName && selectedOperationName !== operationName) {
                operationName = selectedOperationName;
                this.handleEditOperationName(operationName);
            }
            try {
                this.setState({
                    isWaitingForResponse: true,
                    response: undefined,
                    operationName,
                });
                if (this._queryHistory) {
                    this._queryHistory.updateHistory(editedQuery, variables, operationName);
                }
                const subscription = this._fetchQuery(editedQuery, variables, operationName, (result) => {
                    if (queryID === this._editorQueryID) {
                        this.setState({
                            isWaitingForResponse: false,
                            response: GraphiQL.formatResult(result),
                        });
                    }
                });
                this.setState({ subscription });
            }
            catch (error) {
                this.setState({
                    isWaitingForResponse: false,
                    response: error.message,
                });
            }
        };
        this.handleStopQuery = () => {
            const subscription = this.state.subscription;
            this.setState({
                isWaitingForResponse: false,
                subscription: null,
            });
            if (subscription) {
                subscription.unsubscribe();
            }
        };
        this.handlePrettifyQuery = () => {
            const editor = this.getQueryEditor();
            const editorContent = editor?.getValue() ?? '';
            const prettifiedEditorContent = print(parse(editorContent));
            if (prettifiedEditorContent !== editorContent) {
                editor?.setValue(prettifiedEditorContent);
            }
            const variableEditor = this.getVariableEditor();
            const variableEditorContent = variableEditor?.getValue() ?? '';
            try {
                const prettifiedVariableEditorContent = JSON.stringify(JSON.parse(variableEditorContent), null, 2);
                if (prettifiedVariableEditorContent !== variableEditorContent) {
                    variableEditor?.setValue(prettifiedVariableEditorContent);
                }
            }
            catch {
            }
        };
        this.handleMergeQuery = () => {
            const editor = this.getQueryEditor();
            const query = editor.getValue();
            if (!query) {
                return;
            }
            const ast = parse(query);
            editor.setValue(print(mergeAST(ast)));
        };
        this.handleEditQuery = debounce(100, (value) => {
            const queryFacts = this._updateQueryFacts(value, this.state.operationName, this.state.operations, this.state.schema);
            this.setState({
                query: value,
                ...queryFacts,
            });
            if (this.props.onEditQuery) {
                return this.props.onEditQuery(value);
            }
        });
        this.handleCopyQuery = () => {
            const editor = this.getQueryEditor();
            const query = editor && editor.getValue();
            if (!query) {
                return;
            }
            copyToClipboard(query);
            if (this.props.onCopyQuery) {
                return this.props.onCopyQuery(query);
            }
        };
        this._updateQueryFacts = (query, operationName, prevOperations, schema) => {
            const queryFacts = getQueryFacts(schema, query);
            if (queryFacts) {
                const updatedOperationName = getSelectedOperationName(prevOperations, operationName, queryFacts.operations);
                const onEditOperationName = this.props.onEditOperationName;
                if (onEditOperationName &&
                    updatedOperationName &&
                    operationName !== updatedOperationName) {
                    onEditOperationName(updatedOperationName);
                }
                return {
                    operationName: updatedOperationName,
                    ...queryFacts,
                };
            }
        };
        this.handleEditVariables = (value) => {
            this.setState({ variables: value });
            if (this.props.onEditVariables) {
                this.props.onEditVariables(value);
            }
        };
        this.handleEditOperationName = (operationName) => {
            const onEditOperationName = this.props.onEditOperationName;
            if (onEditOperationName) {
                onEditOperationName(operationName);
            }
        };
        this.handleHintInformationRender = (elem) => {
            elem.addEventListener('click', this._onClickHintInformation);
            let onRemoveFn;
            elem.addEventListener('DOMNodeRemoved', (onRemoveFn = () => {
                elem.removeEventListener('DOMNodeRemoved', onRemoveFn);
                elem.removeEventListener('click', this._onClickHintInformation);
            }));
        };
        this.handleEditorRunQuery = () => {
            this._runQueryAtCursor();
        };
        this._onClickHintInformation = (event) => {
            if (event?.currentTarget &&
                'className' in event.currentTarget &&
                event.currentTarget.className === 'typeName') {
                const typeName = event.currentTarget.innerHTML;
                const schema = this.state.schema;
                if (schema) {
                    const type = schema.getType(typeName);
                    if (type) {
                        this.setState({ docExplorerOpen: true }, () => {
                            if (this.docExplorerComponent) {
                                this.docExplorerComponent.showDoc(type);
                            }
                        });
                    }
                }
            }
        };
        this.handleToggleDocs = () => {
            if (typeof this.props.onToggleDocs === 'function') {
                this.props.onToggleDocs(!this.state.docExplorerOpen);
            }
            this.setState({ docExplorerOpen: !this.state.docExplorerOpen });
        };
        this.handleToggleHistory = () => {
            if (typeof this.props.onToggleHistory === 'function') {
                this.props.onToggleHistory(!this.state.historyPaneOpen);
            }
            this.setState({ historyPaneOpen: !this.state.historyPaneOpen });
        };
        this.handleSelectHistoryQuery = (query, variables, operationName) => {
            if (query) {
                this.handleEditQuery(query);
            }
            if (variables) {
                this.handleEditVariables(variables);
            }
            if (operationName) {
                this.handleEditOperationName(operationName);
            }
        };
        this.handleResizeStart = (downEvent) => {
            if (!this._didClickDragBar(downEvent)) {
                return;
            }
            downEvent.preventDefault();
            const offset = downEvent.clientX - getLeft(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                const editorBar = this.editorBarComponent;
                const leftSize = moveEvent.clientX - getLeft(editorBar) - offset;
                const rightSize = editorBar.clientWidth - leftSize;
                this.setState({ editorFlex: leftSize / rightSize });
            };
            let onMouseUp = () => {
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        this.handleResetResize = () => {
            this.setState({ editorFlex: 1 });
        };
        this.handleDocsResizeStart = downEvent => {
            downEvent.preventDefault();
            const hadWidth = this.state.docExplorerWidth;
            const offset = downEvent.clientX - getLeft(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                const app = this.graphiqlContainer;
                const cursorPos = moveEvent.clientX - getLeft(app) - offset;
                const docsSize = app.clientWidth - cursorPos;
                if (docsSize < 100) {
                    this.setState({ docExplorerOpen: false });
                }
                else {
                    this.setState({
                        docExplorerOpen: true,
                        docExplorerWidth: Math.min(docsSize, 650),
                    });
                }
            };
            let onMouseUp = () => {
                if (!this.state.docExplorerOpen) {
                    this.setState({ docExplorerWidth: hadWidth });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        this.handleDocsResetResize = () => {
            this.setState({
                docExplorerWidth: DEFAULT_DOC_EXPLORER_WIDTH,
            });
        };
        this.handleVariableResizeStart = downEvent => {
            downEvent.preventDefault();
            let didMove = false;
            const wasOpen = this.state.variableEditorOpen;
            const hadHeight = this.state.variableEditorHeight;
            const offset = downEvent.clientY - getTop(downEvent.target);
            let onMouseMove = moveEvent => {
                if (moveEvent.buttons === 0) {
                    return onMouseUp();
                }
                didMove = true;
                const editorBar = this.editorBarComponent;
                const topSize = moveEvent.clientY - getTop(editorBar) - offset;
                const bottomSize = editorBar.clientHeight - topSize;
                if (bottomSize < 60) {
                    this.setState({
                        variableEditorOpen: false,
                        variableEditorHeight: hadHeight,
                    });
                }
                else {
                    this.setState({
                        variableEditorOpen: true,
                        variableEditorHeight: bottomSize,
                    });
                }
            };
            let onMouseUp = () => {
                if (!didMove) {
                    this.setState({ variableEditorOpen: !wasOpen });
                }
                document.removeEventListener('mousemove', onMouseMove);
                document.removeEventListener('mouseup', onMouseUp);
                onMouseMove = null;
                onMouseUp = null;
            };
            document.addEventListener('mousemove', onMouseMove);
            document.addEventListener('mouseup', onMouseUp);
        };
        if (typeof props.fetcher !== 'function') {
            throw new TypeError('GraphiQL requires a fetcher function.');
        }
        this._storage = new StorageAPI(props.storage);
        const query = props.query !== undefined
            ? props.query
            : this._storage.get('query')
                ? this._storage.get('query')
                : props.defaultQuery !== undefined
                    ? props.defaultQuery
                    : defaultQuery;
        const queryFacts = getQueryFacts(props.schema, query);
        const variables = props.variables !== undefined
            ? props.variables
            : this._storage.get('variables');
        const operationName = props.operationName !== undefined
            ? props.operationName
            : getSelectedOperationName(undefined, this._storage.get('operationName'), queryFacts && queryFacts.operations);
        let docExplorerOpen = props.docExplorerOpen || false;
        if (this._storage.get('docExplorerOpen')) {
            docExplorerOpen = this._storage.get('docExplorerOpen') === 'true';
        }
        const variableEditorOpen = props.defaultVariableEditorOpen !== undefined
            ? props.defaultVariableEditorOpen
            : Boolean(variables);
        this.state = {
            schema: props.schema,
            query,
            variables: variables,
            operationName,
            docExplorerOpen,
            response: props.response,
            editorFlex: Number(this._storage.get('editorFlex')) || 1,
            variableEditorOpen,
            variableEditorHeight: Number(this._storage.get('variableEditorHeight')) || 200,
            historyPaneOpen: this._storage.get('historyPaneOpen') === 'true' || false,
            docExplorerWidth: Number(this._storage.get('docExplorerWidth')) ||
                DEFAULT_DOC_EXPLORER_WIDTH,
            isWaitingForResponse: false,
            subscription: null,
            ...queryFacts,
        };
        if (typeof window === 'object') {
            window.addEventListener('beforeunload', () => this.componentWillUnmount());
        }
    }
    static formatResult(result) {
        return JSON.stringify(result, null, 2);
    }
    static formatError(rawError) {
        const result = Array.isArray(rawError)
            ? rawError.map(formatSingleError)
            : formatSingleError(rawError);
        return JSON.stringify(result, null, 2);
    }
    componentDidMount() {
        if (this.state.schema === undefined) {
            this.fetchSchema();
        }
        this.codeMirrorSizer = new CodeMirrorSizer();
        global.g = this;
    }
    UNSAFE_componentWillReceiveProps(nextProps) {
        let nextSchema = this.state.schema;
        let nextQuery = this.state.query;
        let nextVariables = this.state.variables;
        let nextOperationName = this.state.operationName;
        let nextResponse = this.state.response;
        if (nextProps.schema !== undefined) {
            nextSchema = nextProps.schema;
        }
        if (nextProps.query !== undefined) {
            nextQuery = nextProps.query;
        }
        if (nextProps.variables !== undefined) {
            nextVariables = nextProps.variables;
        }
        if (nextProps.operationName !== undefined) {
            nextOperationName = nextProps.operationName;
        }
        if (nextProps.response !== undefined) {
            nextResponse = nextProps.response;
        }
        if (nextQuery &&
            nextSchema &&
            (nextSchema !== this.state.schema ||
                nextQuery !== this.state.query ||
                nextOperationName !== this.state.operationName)) {
            const updatedQueryAttributes = this._updateQueryFacts(nextQuery, nextOperationName, this.state.operations, nextSchema);
            if (updatedQueryAttributes !== undefined) {
                nextOperationName = updatedQueryAttributes.operationName;
                this.setState(updatedQueryAttributes);
            }
        }
        if (nextProps.schema === undefined &&
            nextProps.fetcher !== this.props.fetcher) {
            nextSchema = undefined;
        }
        this.setState({
            schema: nextSchema,
            query: nextQuery,
            variables: nextVariables,
            operationName: nextOperationName,
            response: nextResponse,
        }, () => {
            if (this.state.schema === undefined) {
                if (this.docExplorerComponent) {
                    this.docExplorerComponent.reset();
                }
                this.fetchSchema();
            }
        });
    }
    componentDidUpdate() {
        this.codeMirrorSizer.updateSizes([
            this.queryEditorComponent,
            this.variableEditorComponent,
            this.resultComponent,
        ]);
    }
    componentWillUnmount() {
        if (this.state.query) {
            this._storage.set('query', this.state.query);
        }
        if (this.state.variables) {
            this._storage.set('variables', this.state.variables);
        }
        if (this.state.operationName) {
            this._storage.set('operationName', this.state.operationName);
        }
        this._storage.set('editorFlex', JSON.stringify(this.state.editorFlex));
        this._storage.set('variableEditorHeight', JSON.stringify(this.state.variableEditorHeight));
        this._storage.set('docExplorerWidth', JSON.stringify(this.state.docExplorerWidth));
        this._storage.set('docExplorerOpen', JSON.stringify(this.state.docExplorerOpen));
        this._storage.set('historyPaneOpen', JSON.stringify(this.state.historyPaneOpen));
    }
    render() {
        const children = React.Children.toArray(this.props.children);
        const logo = find(children, child => isChildComponentType(child, GraphiQL.Logo)) || React.createElement(GraphiQL.Logo, null);
        const toolbar = find(children, child => isChildComponentType(child, GraphiQL.Toolbar)) || (React.createElement(GraphiQL.Toolbar, null,
            React.createElement(ToolbarButton, { onClick: this.handlePrettifyQuery, title: "Prettify Query (Shift-Ctrl-P)", label: "Prettify" }),
            React.createElement(ToolbarButton, { onClick: this.handleMergeQuery, title: "Merge Query (Shift-Ctrl-M)", label: "Merge" }),
            React.createElement(ToolbarButton, { onClick: this.handleCopyQuery, title: "Copy Query (Shift-Ctrl-C)", label: "Copy" }),
            React.createElement(ToolbarButton, { onClick: this.handleToggleHistory, title: "Show History", label: "History" })));
        const footer = find(children, child => isChildComponentType(child, GraphiQL.Footer));
        const queryWrapStyle = {
            WebkitFlex: this.state.editorFlex,
            flex: this.state.editorFlex,
        };
        const docWrapStyle = {
            display: 'block',
            width: this.state.docExplorerWidth,
        };
        const docExplorerWrapClasses = 'docExplorerWrap' +
            (this.state.docExplorerWidth < 200 ? ' doc-explorer-narrow' : '');
        const historyPaneStyle = {
            display: this.state.historyPaneOpen ? 'block' : 'none',
            width: '230px',
            zIndex: 7,
        };
        const variableOpen = this.state.variableEditorOpen;
        const variableStyle = {
            height: variableOpen ? this.state.variableEditorHeight : undefined,
        };
        return (React.createElement("div", { ref: n => {
                this.graphiqlContainer = n;
            }, className: "graphiql-container" },
            React.createElement("div", { className: "historyPaneWrap", style: historyPaneStyle },
                React.createElement(QueryHistory, { ref: node => {
                        this._queryHistory = node;
                    }, operationName: this.state.operationName, query: this.state.query, variables: this.state.variables, onSelectQuery: this.handleSelectHistoryQuery, storage: this._storage, queryID: this._editorQueryID },
                    React.createElement("button", { className: "docExplorerHide", onClick: this.handleToggleHistory, "aria-label": "Close History" }, '\u2715'))),
            React.createElement("div", { className: "editorWrap" },
                React.createElement("div", { className: "topBarWrap" },
                    React.createElement("div", { className: "topBar" },
                        logo,
                        React.createElement(ExecuteButton, { isRunning: Boolean(this.state.subscription), onRun: this.handleRunQuery, onStop: this.handleStopQuery, operations: this.state.operations }),
                        toolbar),
                    !this.state.docExplorerOpen && (React.createElement("button", { className: "docExplorerShow", onClick: this.handleToggleDocs, "aria-label": "Open Documentation Explorer" }, 'Docs'))),
                React.createElement("div", { ref: n => {
                        this.editorBarComponent = n;
                    }, className: "editorBar", onDoubleClick: this.handleResetResize, onMouseDown: this.handleResizeStart },
                    React.createElement("div", { className: "queryWrap", style: queryWrapStyle },
                        React.createElement(QueryEditor, { ref: n => {
                                this.queryEditorComponent = n;
                            }, schema: this.state.schema, value: this.state.query, onEdit: this.handleEditQuery, onHintInformationRender: this.handleHintInformationRender, onClickReference: this.handleClickReference, onCopyQuery: this.handleCopyQuery, onPrettifyQuery: this.handlePrettifyQuery, onMergeQuery: this.handleMergeQuery, onRunQuery: this.handleEditorRunQuery, editorTheme: this.props.editorTheme, readOnly: this.props.readOnly }),
                        React.createElement("section", { className: "variable-editor", style: variableStyle, "aria-label": "Query Variables" },
                            React.createElement("div", { className: "variable-editor-title", id: "variable-editor-title", style: {
                                    cursor: variableOpen ? 'row-resize' : 'n-resize',
                                }, onMouseDown: this.handleVariableResizeStart }, 'Query Variables'),
                            React.createElement(VariableEditor, { ref: n => {
                                    this.variableEditorComponent = n;
                                }, value: this.state.variables, variableToType: this.state.variableToType, onEdit: this.handleEditVariables, onHintInformationRender: this.handleHintInformationRender, onPrettifyQuery: this.handlePrettifyQuery, onMergeQuery: this.handleMergeQuery, onRunQuery: this.handleEditorRunQuery, editorTheme: this.props.editorTheme, readOnly: this.props.readOnly }))),
                    React.createElement("div", { className: "resultWrap" },
                        this.state.isWaitingForResponse && (React.createElement("div", { className: "spinner-container" },
                            React.createElement("div", { className: "spinner" }))),
                        React.createElement(ResultViewer, { registerRef: n => {
                                this.resultViewerElement = n;
                            }, ref: c => {
                                this.resultComponent = c;
                            }, value: this.state.response, editorTheme: this.props.editorTheme, ResultsTooltip: this.props.ResultsTooltip, ImagePreview: ImagePreview }),
                        footer))),
            this.state.docExplorerOpen && (React.createElement("div", { className: docExplorerWrapClasses, style: docWrapStyle },
                React.createElement("div", { className: "docExplorerResizer", onDoubleClick: this.handleDocsResetResize, onMouseDown: this.handleDocsResizeStart }),
                React.createElement(DocExplorer, { ref: c => {
                        this.docExplorerComponent = c;
                    }, schema: this.state.schema },
                    React.createElement("button", { className: "docExplorerHide", onClick: this.handleToggleDocs, "aria-label": "Close Documentation Explorer" }, '\u2715'))))));
    }
    getQueryEditor() {
        if (this.queryEditorComponent) {
            return this.queryEditorComponent.getCodeMirror();
        }
    }
    getVariableEditor() {
        if (this.variableEditorComponent) {
            return this.variableEditorComponent.getCodeMirror();
        }
        return null;
    }
    refresh() {
        if (this.queryEditorComponent) {
            this.queryEditorComponent.getCodeMirror().refresh();
        }
        if (this.variableEditorComponent) {
            this.variableEditorComponent.getCodeMirror().refresh();
        }
        if (this.resultComponent) {
            this.resultComponent.getCodeMirror().refresh();
        }
    }
    autoCompleteLeafs() {
        const { insertions, result } = fillLeafs(this.state.schema, this.state.query, this.props.getDefaultFieldNames);
        if (insertions && insertions.length > 0) {
            const editor = this.getQueryEditor();
            if (editor) {
                editor.operation(() => {
                    const cursor = editor.getCursor();
                    const cursorIndex = editor.indexFromPos(cursor);
                    editor.setValue(result || '');
                    let added = 0;
                    const markers = insertions.map(({ index, string }) => editor.markText(editor.posFromIndex(index + added), editor.posFromIndex(index + (added += string.length)), {
                        className: 'autoInsertedLeaf',
                        clearOnEnter: true,
                        title: 'Automatically added leaf fields',
                    }));
                    setTimeout(() => markers.forEach(marker => marker.clear()), 7000);
                    let newCursorIndex = cursorIndex;
                    insertions.forEach(({ index, string }) => {
                        if (index < cursorIndex) {
                            newCursorIndex += string.length;
                        }
                    });
                    editor.setCursor(editor.posFromIndex(newCursorIndex));
                });
            }
        }
        return result;
    }
    fetchSchema() {
        const fetcher = this.props.fetcher;
        const fetch = observableToPromise(fetcher({
            query: introspectionQuery,
            operationName: introspectionQueryName,
        }));
        if (!isPromise(fetch)) {
            this.setState({
                response: 'Fetcher did not return a Promise for introspection.',
            });
            return;
        }
        fetch
            .then(result => {
            if (typeof result !== 'string' && result.data) {
                return result;
            }
            const fetch2 = observableToPromise(fetcher({
                query: introspectionQuerySansSubscriptions,
                operationName: introspectionQueryName,
            }));
            if (!isPromise(fetch)) {
                throw new Error('Fetcher did not return a Promise for introspection.');
            }
            return fetch2;
        })
            .then(result => {
            if (this.state.schema !== undefined) {
                return;
            }
            if (typeof result !== 'string' && result && result.data) {
                const schema = buildClientSchema(result.data);
                const queryFacts = getQueryFacts(schema, this.state.query);
                this.setState({ schema, ...queryFacts });
            }
            else {
                const responseString = typeof result === 'string' ? result : GraphiQL.formatResult(result);
                this.setState({
                    schema: undefined,
                    response: responseString,
                });
            }
        })
            .catch(error => {
            this.setState({
                schema: undefined,
                response: error ? GraphiQL.formatError(error) : undefined,
            });
        });
    }
    _fetchQuery(query, variables, operationName, cb) {
        const fetcher = this.props.fetcher;
        let jsonVariables = null;
        try {
            jsonVariables =
                variables && variables.trim() !== '' ? JSON.parse(variables) : null;
        }
        catch (error) {
            throw new Error(`Variables are invalid JSON: ${error.message}.`);
        }
        if (typeof jsonVariables !== 'object') {
            throw new Error('Variables are not a JSON object.');
        }
        const fetch = fetcher({
            query,
            variables: jsonVariables,
            operationName,
        });
        if (isPromise(fetch)) {
            fetch.then(cb).catch(error => {
                this.setState({
                    isWaitingForResponse: false,
                    response: error ? GraphiQL.formatError(error) : undefined,
                });
            });
        }
        else if (isObservable(fetch)) {
            const subscription = fetch.subscribe({
                next: cb,
                error: (error) => {
                    this.setState({
                        isWaitingForResponse: false,
                        response: error ? GraphiQL.formatError(error) : undefined,
                        subscription: null,
                    });
                },
                complete: () => {
                    this.setState({
                        isWaitingForResponse: false,
                        subscription: null,
                    });
                },
            });
            return subscription;
        }
        else {
            throw new Error('Fetcher did not return Promise or Observable.');
        }
    }
    _runQueryAtCursor() {
        if (this.state.subscription) {
            this.handleStopQuery();
            return;
        }
        let operationName;
        const operations = this.state.operations;
        if (operations) {
            const editor = this.getQueryEditor();
            if (editor && editor.hasFocus()) {
                const cursor = editor.getCursor();
                const cursorIndex = editor.indexFromPos(cursor);
                for (let i = 0; i < operations.length; i++) {
                    const operation = operations[i];
                    if (operation.loc &&
                        operation.loc.start <= cursorIndex &&
                        operation.loc.end >= cursorIndex) {
                        operationName = operation.name && operation.name.value;
                        break;
                    }
                }
            }
        }
        this.handleRunQuery(operationName);
    }
    _didClickDragBar(event) {
        if (event.button !== 0 || event.ctrlKey) {
            return false;
        }
        let target = event.target;
        if (target.className.indexOf('CodeMirror-gutter') !== 0) {
            return false;
        }
        const resultWindow = this.resultViewerElement;
        while (target) {
            if (target === resultWindow) {
                return true;
            }
            target = target.parentNode;
        }
        return false;
    }
}
GraphiQL.Logo = GraphiQLLogo;
GraphiQL.Toolbar = GraphiQLToolbar;
GraphiQL.Footer = GraphiQLFooter;
GraphiQL.QueryEditor = QueryEditor;
GraphiQL.VariableEditor = VariableEditor;
GraphiQL.ResultViewer = ResultViewer;
GraphiQL.Button = ToolbarButton;
GraphiQL.ToolbarButton = ToolbarButton;
GraphiQL.Group = ToolbarGroup;
GraphiQL.Menu = ToolbarMenu;
GraphiQL.MenuItem = ToolbarMenuItem;
function GraphiQLLogo(props) {
    return (React.createElement("div", { className: "title" }, props.children || (React.createElement("span", null,
        'Graph',
        React.createElement("em", null, 'i'),
        'QL'))));
}
GraphiQLLogo.displayName = 'GraphiQLLogo';
function GraphiQLToolbar(props) {
    return (React.createElement("div", { className: "toolbar", role: "toolbar", "aria-label": "Editor Commands" }, props.children));
}
GraphiQLToolbar.displayName = 'GraphiQLToolbar';
function GraphiQLFooter(props) {
    return React.createElement("div", { className: "footer" }, props.children);
}
GraphiQLFooter.displayName = 'GraphiQLFooter';
const formatSingleError = (error) => ({
    ...error,
    message: error.message,
    stack: error.stack,
});
const defaultQuery = `# Welcome to GraphiQL
#
# GraphiQL is an in-browser tool for writing, validating, and
# testing GraphQL queries.
#
# Type queries into this side of the screen, and you will see intelligent
# typeaheads aware of the current GraphQL type schema and live syntax and
# validation errors highlighted within the text.
#
# GraphQL queries typically start with a "{" character. Lines that start
# with a # are ignored.
#
# An example GraphQL query might look like:
#
#     {
#       field(arg: "value") {
#         subField
#       }
#     }
#
# Keyboard shortcuts:
#
#  Prettify Query:  Shift-Ctrl-P (or press the prettify button above)
#
#     Merge Query:  Shift-Ctrl-M (or press the merge button above)
#
#       Run Query:  Ctrl-Enter (or press the play button above)
#
#   Auto Complete:  Ctrl-Space (or just start typing)
#

`;
function isPromise(value) {
    return typeof value === 'object' && typeof value.then === 'function';
}
function observableToPromise(observable) {
    if (!isObservable(observable)) {
        return observable;
    }
    return new Promise((resolve, reject) => {
        const subscription = observable.subscribe(v => {
            resolve(v);
            subscription.unsubscribe();
        }, reject, () => {
            reject(new Error('no value resolved'));
        });
    });
}
function isObservable(value) {
    return (typeof value === 'object' &&
        'subscribe' in value &&
        typeof value.subscribe === 'function');
}
function isChildComponentType(child, component) {
    if (child?.type?.displayName &&
        child.type.displayName === component.displayName) {
        return true;
    }
    return child.type === component;
}
//# sourceMappingURL=GraphiQL.js.map